

unsigned int pdlDotLength_usec;
unsigned int pdlDashLength_usec;
unsigned int pdlElSpace_usec;  
unsigned int pdlCharSpace_usec;
unsigned int pdlWordSpace_usec;
                              
unsigned int kbdDotLength_usec;
unsigned int kbdDashLength_usec;
unsigned int kbdElSpace_usec;  
unsigned int kbdCharSpace_usec;
unsigned int kbdWordSpace_usec;

/** -------------------------------------------------------------------------
I measured the element length for a dot on the scope and it appears to be on the
fast side, about 3.33%.  I meaure a 20 msec dot length when I set the speed at 58 wpm.

I will use a constant try to adjust for this:  1,200,000 * 1.0333 = 1,239,960 to
"slow it down" about 3.33%.

Much later:  I measured again and seem to be slightly slow, so I am now trying
1,200,000 * 1.025 = 1,230,000.  I think I can see the increase in speed, but need a bit more:

1,200,000 * 1.015 = 1,218,000

I finally used the counter to measure period, and the 1,200,000 value seems to be right on:

WPM Setting	Period msecs	WPM calculated
99	           24.254	         98.95
90	           26.678            89.96
80	           30.012	         79.97	
70	           34.296	         69.98
60	           40.012	         59.98
50	           48.012	         49.99
40	           60.012	         39.99
30	           80.012	         30.00
20	           120.012	         20.00

I also checked weight accuracy and it is essentially perfect also.  All of this at the original
1,200,000 fundamental parameter.

------------------------------------------------------------------------------ **/

unsigned int speedConstantTweak = 1200000;  // 1218000;  //1230000;  // 1239960;


/** -------------------------------------------------------------------------
@verbatim
Calculations for Paddle Keyer Timing:
Calculate element and space durations in units of  usec. 

The fundamental equation from which we work relates WPM to the frequency of a
string of dots.  e.g. at 60 wpm a dot duration is 20 msec and the dot period
is 40 msec = 25 Hz.

We begin with this equation:  WPM =           2.4 * dotHz
                              dotHz =         WPM / 2.4
                              dotPeriodSec =  2.4 / WPM
                              dotPeriodMsec = 2400 / WPM

Now, the period in units of  usec becomes:
                              dotPeriod_usec  = 2400000 / WPM

and finally, we calculate the basic dot length in units of usec:
                              dotLength_usec = 1200000 / WPM


NOTE:  wordSpace is not used for the paddle keyer cw generation <--********

To include WEIGHT in the calculation:

Given the weight setting specified as the DOT duty cycle percentage for a string
of dits (nominally 50% where the dot duration equals the duration of the element
space).  If the weight is set to 45% (fairly light weight), then the dot duration 
equals 45% of the dot period while the element space duration equals 55% of the
dot period.

For example, at 60 WPM and 50% weight:  dot = 20 msec  space = 20 msec
At 60 WPM and 45% weight:               dot = 18 msec  space = 22 msec
where the delta subtracted from the nominal dot and added to the nominal
space is 2 msec = 5% of the dot period or 10% of the dot duration.

We calculate the delta to be added/subtracted to the dot/space by subtracting
the specified weight from 50.

If the result is positive: 
   We need to shorten the dot and lengthen the space (light weight).
   Otherwise we lengthen the dot and shorten the space (heavy weight).

This routine calculates these global parameters as a function of paddle speed in WPM:

pdlDotLength_usec;
pdlDashLength_usec
pdlElSpace_usec;  
pdlCharSpace_usec;
pdlWordSpace_usec;
                  
@endverbatim

@param [in] wpm  Speed setting
@param [in] weight  Weight (nominally 50%)
------------------------------------------------------------------------------ **/
void calcTimeDurationsPdl(unsigned char wpm, unsigned char weight)
{
  int deltaPercent;  
  int delta_usecCount;

  //First, calculate the dot length assumming weight = 50%
  pdlDotLength_usec  = speedConstantTweak / wpm;
  pdlElSpace_usec    = pdlDotLength_usec;
  delta_usecCount = 0;


  if (weight != 50)
    {
      // We will calculate the percentage of the dot duration that we need to either
      // shorten or lengthen the dot duration by.  This is twice the difference between
      // 50% and the specified weight:
      deltaPercent = 2*(50 - weight);

      // Use integer arithmetic (multiply first, then divide) to prevent floating
      // point code to be linked in.  Saves something like 3K bytes.
      delta_usecCount = (deltaPercent * (int) pdlDotLength_usec) / 100;

      // If the weight is less than 50%, delta_usecCount will be positive, so we SUBTRACT
      // this value to the dotLength_usec value and ADD it to the space.
    }


  pdlDotLength_usec -= delta_usecCount;
  pdlElSpace_usec   += delta_usecCount;

  pdlDashLength_usec = 3*pdlDotLength_usec;

  // Character space calculation are not actually used for the paddle keyer mode!
  pdlCharSpace_usec  = pdlDashLength_usec;

  // Word space for paddle mode is only used to detect an intended word space.
  // Since a word space is 7 element spaces and we are now two element spaces past the
  // last element, we can set the word space timer to perhaps 3 or 4 more element spaces, 
  // making 5 total out of 7.  Let's try using 4 initially.  
  pdlWordSpace_usec  = 4*pdlElSpace_usec;

}


/** -------------------------------------------------------------------------
@verbatim
Calculations for Keyboard Keyer Timings
Same as calcTimeDurationsPdl except for the variables affected.

NOTE:  The word space calculation would normally be 7 times the kbdDotLength but
       the cw generator provides an automatic character space (3 units) so the
       actual calculation for word space is only 4 * kbdDotLength

       ALSO:  Since each element is automatically followed by an element space,
              this means the timer setting for character spaces needs to by set
              to 2 units instead of 3.

@endverbatim

@param [in] wpm  Speed setting
@param [in] weight  Weight (nominally 50%)
------------------------------------------------------------------------------ **/

void calcTimeDurationsKbd(unsigned char wpm, unsigned char weight)
{
  int deltaPercent;  
  int delta_usecCount;

  //First, calculate the dot length assumming weight = 50%
  kbdDotLength_usec  = speedConstantTweak / wpm;
  kbdElSpace_usec    = kbdDotLength_usec;
  delta_usecCount = 0;


  if (weight != 50)
    {
      // We will calculate the percentage of the dot duration that we need to either
      // shorten or lengthen the dot duration by.  This is twice the difference between
      // 50% and the specified weight:
      deltaPercent = 2*(50 - weight);

      // Use integer arithmetic (multiply first, then divide) to prevent floating
      // point code to be linked in.  Saves something like 3K bytes.
      delta_usecCount = (deltaPercent * (int) kbdDotLength_usec) / 100;

      // If the weight is less than 50%, delta_usecCount will be positive, so we SUBTRACT
      // this value to the dotLength_usec value and ADD it to the space.
    }


  kbdDotLength_usec -= delta_usecCount;
  kbdElSpace_usec   += delta_usecCount;


  kbdDashLength_usec = 3*kbdDotLength_usec;

  // We only use 2 units for character space because each element is
  // automatically followed by one unit space for the element space.
  kbdCharSpace_usec  = 2*kbdElSpace_usec;


  // We only use four units for word space instead of 7
  // because each character is automatically followed by
  // the 3 unit character space.  Thus a subsequent word
  // space only needs to add an additional 4 units of space.
  kbdWordSpace_usec  = 4*kbdElSpace_usec;

}
