#include "mbed.h"
#include "w5uxhMbedKeyer.h"
#include "queues.h"
#include <iostream>
#include <fstream>
#include "ConfigFile.h"

extern unsigned char pdlWPM;
extern unsigned char kbdWPM;
extern unsigned char weight;
extern char idString[];
extern Serial pc;
extern Fifo *cwBuf;
extern char kbdCwState;

static char filename[9][20] =
  {
    "/keyer/Msg1.txt", "/keyer/Msg2.txt", "/keyer/Msg3.txt", 
    "/keyer/Msg4.txt", "/keyer/Msg5.txt", "/keyer/Msg6.txt", 
    "/keyer/Msg7.txt", "/keyer/Msg8.txt", "/keyer/Msg9.txt"
  };

LocalFileSystem keyer("keyer");


/** -------------------------------------------------------------------------
@verbatim
The control code obtained from the scan code to "ascii code" lookup is passed
to this function so we can identify which keypad message key was pressed.

The codes are 0xA8 through 0xB0 for keypad numeric keys 1-9.

@endverbatim
**/
void sendCannedMsg(char asciiChar)
{

  long begin, end;
  int fileSize;
  char inChar;
  int i;
  int filenameIndex;
  bool bufWriteStatus=FALSE;

  // Generate the index 0-8 to use to lookup the filename
  filenameIndex = asciiChar - 0xA8;
  
  ifstream in(filename[filenameIndex], ios::binary|ios::in);
  
  if (!in)
    {
      pc.printf("\runable to open file\r");
      return;
    }
  
  begin = in.tellg();
  in.seekg(0, ios::end);
  end = in.tellg();
  fileSize = (end-begin);
  //  pc.printf("file size = %d\r", fileSize);
  
  in.seekg(0, ios::beg);
  
  pc.printf(" ");
  cwBuf->write(CANNED_MSG_START);
  for(i=0; i<fileSize; i++)
    {
      in.get(inChar);
      if ((inChar != 0x0D) && (inChar != 0x0A))
      {
        bufWriteStatus = cwBuf->write(inChar);
      }
      else
      {
        cwBuf->write(' ');
      }
      if (inChar != 0x0A)
      {
        if (bufWriteStatus) pc.printf("%c", inChar);
      }
      if (kbdCwState == KBD_IDLE)
        {
          checkCwBuffer();
        }
    }
    
      //  Pad the message with a single space at the end
  cwBuf->write(' ');
  cwBuf->write(CANNED_MSG_END);  
  in.close();
  pc.printf(" ");
}

void previewMsg(char asciiChar)
{
  long begin, end;
  int fileSize;
  char inChar;
  int i;
  int filenameIndex;
  int msgNr;
  char tempString[6];

  // Generate the index 0-8 to use to lookup the filename
  filenameIndex = asciiChar - 0xE8;
  msgNr = filenameIndex + 1;

  ifstream in(filename[filenameIndex], ios::binary|ios::in);
  
  if (!in)
    {
      pc.printf("\runable to open file\r");
      return;
    }
  
  begin = in.tellg();
  in.seekg(0, ios::end);
  end = in.tellg();
  fileSize = (end-begin);
  //  pc.printf("file size = %d:\r", fileSize);
  
  in.seekg(0, ios::beg);
  pc.printf("\rPreview Msg# %d-->   ", msgNr);
  clearLcdRow(2);

  for(i=0; i<fileSize; i++)
    {
      in.get(inChar);
      pc.printf("%c", inChar);
      if(i<13) displayLcdTextRow(2, inChar);
      if(i==13) displayLcdStringRow(2, "...");
    }

  in.close();

  if (fileSize > 20) 
  {
    pc.printf("\rEnd of Preview Msg# %d\r", msgNr);
  }
  else
  {
    pc.printf("\r");
  }

  sprintf(tempString, " :%d ", msgNr);
  displayLcdStringRow(2, tempString);
}

ConfigFile cfg;
char buf_key[BUFSIZ];
char buf_value[BUFSIZ];


#define NR_OF_KEYS 7

// IMPORTANT:  The defines below must match the index of the keys in the array:
#define ID_KEY     0
#define KBDSPD_KEY 1
#define PDLSPD_KEY 2
#define WT_KEY     3
#define RIGENA_KEY 4
#define MONENA_KEY 5
#define CURTIS_KEY 6

//  Indices:                 0          1         2         3        4        5         6
char keys[NR_OF_KEYS][10]={"ID",  "kbdspd", "pdlspd", "weight","rigena","monena", "curtis"};

extern bool rigEna, monEna;
extern unsigned char curtisType;
char ena_dis_strings[2][6] = {"OFF", "ON"};
char curtis_strings[2][7]  = {"TYPE_A", "TYPE_B"};

int fileInputSettings(void)
{
  int keyCount;

  if (!cfg.read("/keyer/Settings.txt"))
    {
      pc.printf("Failure to read Settings.txt.\r");
      return -1;
    }
  else
    {
      pc.printf("Settings read from:   Settings.txt\r");
    }

  const int cnt = cfg.getCount();

  for (int i = 0; i < cnt; i++) 
    {
      if (cfg.getKeyAndValue(i, buf_key, sizeof(buf_key), buf_value, sizeof(buf_value))) 
        {
      //  pc.printf("No.%3d:'%s'='%s'\r", i, buf_key, buf_value);
      for (keyCount=0; keyCount<NR_OF_KEYS; keyCount++)
        {
          if(strcmp(buf_key, keys[keyCount])==0) break;
        }
      if (keyCount < NR_OF_KEYS)
        {
          switch (keyCount)
        {
        case ID_KEY:
          {
            strcpy(idString,buf_value);
            pc.printf("   ID         = %s\r", idString);
            break;
          }
        case PDLSPD_KEY:
          {
            pdlWPM = atoi (buf_value);
            pc.printf("   PDL Speed  = %2d\r", pdlWPM);
            break;
          }
        case KBDSPD_KEY:
          {
            kbdWPM = atoi(buf_value);
            pc.printf("   KBD Speed  = %2d\r", kbdWPM);
            break;
          }
        case WT_KEY:
          {
            weight = atoi (buf_value);
            pc.printf("   Weight     = %2d\r", weight);
            break;
          }
        case RIGENA_KEY:
          {
            rigEna = atoi (buf_value);
            pc.printf("   RIG ENABLE = %s\r", ena_dis_strings[rigEna]);
            break;
          }
        case MONENA_KEY:
          {
            monEna = atoi (buf_value);
            pc.printf("   MON ENABLE = %s\r", ena_dis_strings[monEna]);
            break;
          }
        case CURTIS_KEY:
          {
            curtisType = atoi (buf_value);
            pc.printf("   CurtisType = %s\r", curtis_strings[curtisType]);
            break;
          }
        }
        }
      else
        {
          pc.printf("no match: %s  %s\r", buf_key, buf_value);
        }
        } 
      else 
        {
      pc.printf("No.%3d:Failure to get a configuration.\r", i);
        }
    }
  pc.printf("\r");
  displaySpeedsSerial();
  return 0;
}

int fileOutputSettings(void)
{
  bool status=TRUE;
  char paramString[10];

  if (!cfg.setValue("ID", idString )) 
    {
      pc.printf("\r%s\r", "Failure to set ID string in config.\r");  
    }

  sprintf(paramString, "%d", pdlWPM);
  if (!cfg.setValue("pdlspd", paramString)) 
    {
      pc.printf("\r%s\r", "Failure to set pdlWPM in config.\r");  
    }

  sprintf(paramString, "%d", kbdWPM);
  if (!cfg.setValue("kbdspd", paramString)) 
    {
      pc.printf("\r%s\r", "Failure to set kbdWPM in config.\r");  
    }

  sprintf(paramString, "%d", weight);
  if (!cfg.setValue("weight", paramString)) 
    {
      pc.printf("\r%s\r", "Failure to set weight in config.\r");  
    }

  sprintf(paramString, "%d", rigEna);
  if (!cfg.setValue("rigena", paramString)) 
    {
      pc.printf("\r%s\r", "Failure to set rigEna in config.\r");  
    }

  sprintf(paramString, "%d", monEna);
  if (!cfg.setValue("monena", paramString)) 
    {
      pc.printf("\r%s\r", "Failure to set monEna in config.\r");  
    }

  sprintf(paramString, "%d", curtisType);
  if (!cfg.setValue("curtis", paramString)) 
    {
      pc.printf("\r%s\r", "Failure to set curtisType in config.\r");  
    }



  if (status)
    {
      if (!cfg.write("/keyer/Settings.txt"))
    {
      pc.printf("\rFailure to write Settings.txt.\r");
      return -1;
    }
      else
    {
      pc.printf("\rSettings written to:   Settings.txt\r");
    }
      return 0;
    }
  else
    {
      return -1;
    }
}
